
/******************************************************************************
* MODULE     : mmlout.hpp
* DESCRIPTION: Standard output facilities and switches
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_MMLOUT_HPP
#define __MML_MMLOUT_HPP
#include "base.hpp"
namespace mml {

/******************************************************************************
* Printing lisp expressions
******************************************************************************/

template<typename C> ostream&
print_lisp (ostream& out, C x) {
  return out << x;
}

template<typename T> ostream&
print_lisp (ostream& out, char* fun, const T& x) {
  out << "(" << fun << " ";
  print_lisp (out, x);
  return out << ")";
}

template<typename T, typename U> ostream&
print_lisp (ostream& out, char* fun, const T& x, const U& y) {
  out << "(" << fun << " ";
  print_lisp (out, x);
  out << " ";
  print_lisp (out, y);
  return out << ")";
}

/******************************************************************************
* Output switches
******************************************************************************/

template<typename T>
class output_switch {
public:
  T& var;
  T  val;
  output_switch (T& var2, T val2): var (var2), val (val2) {}
};

template<typename T> ostream&
operator << (ostream& o, const output_switch<T>& x) {
  x.var= x.val; return o;
}

extern nat   the_series_prec;     // series precision for printing
extern nat   the_series_zt;       // series precision for zero-testing
extern char* the_series_var;      // name for the series variable
extern bool  the_series_expr;     // display series as expressions
extern char* the_polynomial_var;  // name for the polynomial variable

inline output_switch<nat> set_series_prec (nat n= 10) {
  return output_switch<nat> (the_series_prec, n); }
inline output_switch<nat> set_series_zt (nat n= 25) {
  return output_switch<nat> (the_series_zt, n); }
inline output_switch<char*> set_series_var (char* name= "z") {
  return output_switch<char*> (the_series_var, name); }
inline output_switch<bool> set_series_expr (bool flag= false) {
  return output_switch<bool> (the_series_expr, flag); }
inline output_switch<char*> set_polynomial_var (char* name= "x") {
  return output_switch<char*> (the_polynomial_var, name); }

} // namespace mml
#endif // __MML_MMLOUT_HPP
