
/******************************************************************************
* MODULE     : interval_extend.cpp
* DESCRIPTION: Extend the precision of an interval
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "interval_impl.hpp"
namespace mml {

interval
extend (const interval& x, mp_prec_t new_prec) {
  if ((prec (x) >= new_prec) || (new_prec <= DEFAULT_LOW)) return x;
  else {
    interval r= raw_interval (new_prec);
    if (has_low_prec (x)) mpfr_mid (r->mid, x->min, x->max, GMP_RNDN);
    else mpfr_set (r->mid, x->mid, GMP_RNDN);
    if (mpfr_sgn (r->mid) == 0)
      fatal_error ("Cannot extend zero centered interval",
		   "extend", "interval.cpp");
    mpfr_set_ui_2si (r->rad, 1, mpfr_get_step (r->mid), GMP_RNDU);
    return r->finish_high ();
  }
}

} // namespace mml
