
/******************************************************************************
* MODULE     : polynomial.hpp
* DESCRIPTION: Generic univariate polynomials
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_POLYNOMIAL_HPP
#define __MML_POLYNOMIAL_HPP
#include "iterator.hpp"
#include "string.hpp"
namespace mml {
#define TMPL_DEF template<typename C, typename V=dense<naive> >
#define TMPL template<typename C, typename V>
#define Polynomial polynomial<C,V>
TMPL class polynomial;

/******************************************************************************
* Abstract specification of dense polynomials
******************************************************************************/

TMPL_DEF class polynomial_rep REP_STRUCT {};

TMPL_DEF
class polynomial {
INDIRECT_PROTO_2 (polynomial, polynomial_rep, C, V)
public:
  polynomial ();
  polynomial (int x);
  polynomial (const C& x, nat n=0);
  polynomial (const iterator<C>& it);
  C operator[] (nat i);
};
INDIRECT_IMPL_2 (polynomial, polynomial_rep, typename C, C, typename V, V)

TMPL int deg (const Polynomial& P);

/******************************************************************************
* Polynomial iteration
******************************************************************************/

TMPL_DEF
class polynomial_iterator_rep: public iterator_rep<C> {
  Polynomial P;  // the polynomial to traverse
  nat i;         // current position
protected:
  bool is_busy () { return ((int) i) <= deg (P); }
  void advance () { i++; }
  C current () { return P[i]; }
public:
  polynomial_iterator_rep (const Polynomial& P2): P(P2), i(0) {}
};

TMPL iterator<C>
iterate (const Polynomial& P) {
  return iterator<C> (new polynomial_iterator_rep<C,V> (P), false);
}

/******************************************************************************
* Generic output routines for dense polynomials
******************************************************************************/

TMPL string var (const Polynomial& P) {
  (void) P; return the_polynomial_var; }

TMPL ostream&
print_lisp (ostream& out, const Polynomial& P) {
  int i, len= deg(P);
  if (len < 0) return out << "0";
  out << "(+";
  for (i=len; i>=0; i--)
    if (P[i] != 0) {
      out << " (* ";
      (void) print_lisp (out, P[i]);
      out << " (^ " << var (P) << " " << i << "))";
    }
  out << ")";
  return out;
}

TMPL ostream&
operator << (ostream& out, const Polynomial& P) {
  bool first= true;
  int i, len= deg(P);
  if (len<0) return out << "0";
  for (i=len; i>=0; i--)
    if (P[i] != 0) {
      if (!first) out << " + ";
      if (i == 0) out << P[i];
      else {
	if (P[i] == -1) out << "-";
	else if (P[i] != 1) out << P[i] << " ";
	out << var (P);
	if (i > 1) out << "^" << i;
      }
      first= false;
    }
  return out;
}

#undef TMPL_DEF
#undef TMPL
#undef Polynomial
} // namespace mml
#endif // __MML_POLYNOMIAL_HPP
